/*-------------------------------------------------------------------
    All files in this archive are under the GNU GPL v3 license
   Copyright 2008-2009 Purdea Andrei purdeaandrei _at_ yahoo.com
---------------------------------------------------------------------
  
  shiftreg.v
  
  Description: This is the internal shift register used to send
  the data out to the external shift register.
  
  This is a parallel-in serial-out shift register. At every positive
  clock edge either a new value is loaded into the register or a bit
  is shifted out of it. This is selected by a synchronous ``load''
  input. The bit to be shifted out appears on the ``sout'' output,
  and the bit to be shifted in should appear in the ``sin'' input.

-------------------------------------------------------------------*/

module shiftreg(clk, load, data, sin, sout);
    parameter dw = 8; // Data Width parameter.
    input clk;
    input load; // if = 1, then load, otherwise shift
    input [dw-1 : 0] data; // parallel data input
    input sin; // serial data in for daisychaining the registers
    output sout; // serial data out, will contain MSB

    reg [dw-1 : 0] saved; // this will store the data to be shifted out

    initial saved = 0;
    always @(posedge clk) begin
        if (load)
            saved <= data;
        else
            saved <= {sin, saved[dw-1:1]};
    end

    assign sout = saved[0];

endmodule
