/*-------------------------------------------------------------------
    All files in this archive are under the GNU GPL v3 license
   Copyright 2008-2009 Purdea Andrei purdeaandrei _at_ yahoo.com
---------------------------------------------------------------------

  timer_top.v

  Description: This toplevel module implements a hexadecimal timer.
  For example, if it shows 12e.8 seconds, then 302.5 seconds have
  elapsed.

  It is controlled by two buttons. When SW0 is on the timer is
  counting up. When SW0 is off the timer stops counting and
  displays the last value of the counter.
  When SW1 is on the timer resets to zero.

-------------------------------------------------------------------*/
module top(clk, stclk, shclk, sout, sw);
input clk; // 50mHz clock on the s3e starter kit board.
output stclk, shclk, sout; // wired to the extension board
input [3:0] sw; // the status of the switches. only sw0 and sw1 are used.

wire [31:0] data; // this contains the 7-seg decoded value shown on the display

reg [16:0] timereg; // this contains the binary value shown on the display.

genvar i;
generate
    for (i=0;i<4;i=i+1) begin : bla
        hexdecoder h0 (.i(timereg[(3-i+1)*4-1:(3-i)*4]), .o(data[(i+1) * 8 - 1:i * 8 + 1]));
    end
endgenerate

// these are the dots:
assign data[ 0] = 0;
assign data[ 8] = 0;
assign data[16] = 1;
assign data[24] = 0;

extdriver e (.clk50(clk), .stclk(stclk), .shclk(shclk), .sout(sout), 
   .data(data));

reg [21:0] divider; // used for dividing the 50 mhz clock

initial begin
    timereg = 0;
    divider = 0;
end

always @(posedge clk) begin
    if (divider == 0) begin
        divider <= 3125000; // 50mHz div 3125000 = 15 Hz
        if (sw[0]) // is the timer running?
            timereg <= timereg + 1;
    end else begin
        divider <= divider - 1;
    end
    if (sw[1]) timereg <= 0; // timer reset?
end

endmodule